//----------------------------------------------------------------------------------
// File:   DX10FeatureDemo.fx
// Author: Andrew Tatarinov
// Email:  sdkfeedback@nvidia.com
// 
// Copyright (c) 2007 NVIDIA Corporation. All rights reserved.
//
// TO  THE MAXIMUM  EXTENT PERMITTED  BY APPLICABLE  LAW, THIS SOFTWARE  IS PROVIDED
// *AS IS*  AND NVIDIA AND  ITS SUPPLIERS DISCLAIM  ALL WARRANTIES,  EITHER  EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED  TO, IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL  NVIDIA OR ITS SUPPLIERS
// BE  LIABLE  FOR  ANY  SPECIAL,  INCIDENTAL,  INDIRECT,  OR  CONSEQUENTIAL DAMAGES
// WHATSOEVER (INCLUDING, WITHOUT LIMITATION,  DAMAGES FOR LOSS OF BUSINESS PROFITS,
// BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR ANY OTHER PECUNIARY LOSS)
// ARISING OUT OF THE  USE OF OR INABILITY  TO USE THIS SOFTWARE, EVEN IF NVIDIA HAS
// BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
//
//
//----------------------------------------------------------------------------------

// Variables

float4 EyePosition;
float4 LightPosition;
float4 LightColor;
float4 AmbientColor;

int ParallelSlices;
int MeridianSlices;
float TorusKnotP;
float TorusKnotQ;

float Time;

// Textures

Texture2DArray DiffTextureArray;
Texture2DArray BumpTextureArray;
Texture2DArray ShadowMap;
Texture2D ShadowAtlas;

Texture2D<uint> PermTexture;   // permutation texture

Buffer BoxVertexBuffer;
Buffer BoxTexCoordBuffer;

// Texture samples

SamplerState SamplerLinear
{
	Filter = MIN_MAG_MIP_LINEAR;
	AddressU = Wrap;
	AddressV = Wrap;
};

SamplerState SamplerCube
{
	Filter = MIN_MAG_MIP_LINEAR;
	AddressU = Clamp;
	AddressV = Clamp;
};

// Defines

#define M_PI 3.14159265358979323846f

// Transformation matrices

matrix ViewMatrix;
matrix ProjectionMatrix;
matrix CubeViewMatrices[6];
matrix CubeProjectionMatrix;

// Rendering torus knot
// Some structures

struct VSTorus_Input
{
    row_major float4x4	WorldTransform	: WorldTransform;
    uint				VertexId		: SV_VertexID;
    uint				InstanceId		: SV_InstanceID;
};

struct GSTorus_Input
{
    float4	Pos			: SV_POSITION;
    float3	Norm		: TEXCOORD0;
    float3	Tex			: TEXCOORD1;
};

struct PSTorus_Input
{
    float4	Pos			: SV_POSITION;
    float3	Norm		: TEXCOORD0;
    float3	Tex			: TEXCOORD1;
    float4	RealPos		: TEXCOORD2;
};

// Functions for torus knot generations
// Get a point on a torus knot curve

float4 GetTorusKnotCenter( float phi, int theta )
{
    float4 pos = float4( 0.0f, 0.0f, 0.0f, 1.0f );
    float p = TorusKnotP, q = TorusKnotQ;
    
    // Compute a position on a curve using torus knot equation

    pos.x = (2.0f + cos( q * phi / p )) * cos( phi );
    pos.y = (2.0f + cos( q * phi / p )) * sin( phi );
    pos.z = sin( q * phi / p );

    return pos;
}

// Get a point on a torus knot surface

float4 GetTorusKnotPos( float phi, int theta )
{
    float4 pos = float4( 0.0f, 0.0f, 0.0f, 1.0f );
    float3 axisX = float3( 0.0f, 0.0f, 0.0f ), axisY = float3( 0.0f, 0.0f, 0.0f );
    float p = TorusKnotP, q = TorusKnotQ;
    float3 d = float3( 0.0f, 0.0f, 0.0f ), d2 = float3( 0.0f, 0.0f, 0.0f );
    
    // Compute a position on a curve using torus knot equation

    pos.x = (2.0f + cos( q * phi / p )) * cos( phi );
    pos.y = (2.0f + cos( q * phi / p )) * sin( phi );
    pos.z = sin( q * phi / p );
    
    // Since we want to generate not just a curve, but a thick torus knot, we need not only to compute a point on a 
    // curve (a curve is like a cylinder axis), but also find a normal to this curve in given point
    // (whic is like a cylinder radius). We can compute a normal by computing
    // a second derivative to our curve.

    // Compute a first derivative for a torus knot equation.
    // It would be a tangent to a curve

    d.x = -(2.0f + cos( q * phi / p )) * sin( phi ) + cos( phi ) * (2.0f - sin( q * phi / p) * q / p);
    d.y = (2.0f + cos( q * phi / p )) * cos( phi ) + sin( phi ) * (2.0f - sin( q * phi / p) * q / p);
    d.z = cos( q * phi / p ) * q / p;
    
    // Compute a second derivative for a torus knot equation

    d2.x = -d.y - sin( phi ) * (2.0f - sin( q * phi / p ) * q / p) + cos( phi ) * (2.0f - cos( q * phi / p ) * q * q / p / p);
    d2.x = d.x + cos( phi ) * (2.0f - sin( q * phi / p ) * q / p) + sin( phi ) * (2.0f - cos( q * phi / p ) * q * q / p / p);
    d2.z = -sin( q * phi / p ) * q * q / p / p;

    // So, we've got a normal, and we need to sweep it around torus knot curve to build our shape.
    
    axisY = normalize( cross( d, d2 ) );
    axisX = normalize( cross( d, axisY ) );

    pos = pos + (float4( axisX, 0.0f ) * cos( theta / (float)MeridianSlices * 2.0f * M_PI ) * 0.7f + 
                 float4( axisY, 0.0f ) * sin( theta / (float)MeridianSlices * 2.0f * M_PI ) * 0.7f);

    return pos;
}

// Calculate point's texture coordinates on a torus knot

float2 GetTorusKnotTex( int parallel, int meridian )
{
	float2 tex = float2( 0.0f, 0.0f );

	tex.x = (MeridianSlices - meridian) / ((float)MeridianSlices);
	tex.y = (ParallelSlices - parallel) / ((float)ParallelSlices) * 10.0f;

	return tex;
}

// Vertex Shader

GSTorus_Input VSTorus( VSTorus_Input input )
{
    GSTorus_Input output = (GSTorus_Input) 0;
    float4 nearbyPoints[4], torusCenter;
    
    // Use VertexId and integer math operations to get x- and y-indices os a point on a torus knot.
    // X-coordinate goes across a torus knot curve, while y-coordinate is swept aroung this curve. 

    int parallel = input.VertexId % (ParallelSlices + 1),
        meridian = input.VertexId / (ParallelSlices + 1);
    
    // Get a point on a torus knot curve

    torusCenter = GetTorusKnotCenter( parallel / (float)ParallelSlices * 2.0f * M_PI * TorusKnotP, meridian );
    
    // Get a point on a torus knot surface and compute texture coordinates on this surface

    output.Pos = GetTorusKnotPos( parallel / (float)ParallelSlices * 2.0f * M_PI * TorusKnotP, meridian );
    output.Tex.xy = GetTorusKnotTex( parallel, meridian );
    output.Tex.z = (float)input.InstanceId;
    
    // Use texture fetch in VS to shift a point on a torus knot surface according to a value in displacement map

    output.Pos = torusCenter + (output.Pos - torusCenter) * 0.3f + (output.Pos - torusCenter) * 0.8f * 
                 (BumpTextureArray.SampleLevel( SamplerLinear, output.Tex, 0 ));
    output.Pos = mul( output.Pos, input.WorldTransform );
    
    // Use the same approach to compute four nearby points, which we will use to find a normal to
    // a bumped torus knot surface

    nearbyPoints[0] = GetTorusKnotPos( parallel / (float)ParallelSlices * 2.0f * M_PI * TorusKnotP, meridian - 1 );
    nearbyPoints[0] = torusCenter + (nearbyPoints[0] - torusCenter) * 0.3f + (nearbyPoints[0] - torusCenter) * 0.8f * 
                      (BumpTextureArray.SampleLevel( SamplerLinear, float3( GetTorusKnotTex( parallel, meridian - 1 ), (float)input.InstanceId ), 0 ));
    nearbyPoints[0] = mul( nearbyPoints[0], input.WorldTransform );

    nearbyPoints[1] = GetTorusKnotPos( (parallel + 1) / (float) ParallelSlices * 2.0f * M_PI * TorusKnotP, meridian );
    nearbyPoints[1] = torusCenter + (nearbyPoints[1] - torusCenter) * 0.3f + (nearbyPoints[1] - torusCenter) * 0.8f * 
                      (BumpTextureArray.SampleLevel( SamplerLinear, float3( GetTorusKnotTex( parallel + 1, meridian ), (float)input.InstanceId ), 0 ));
    nearbyPoints[1] = mul( nearbyPoints[1], input.WorldTransform );

    nearbyPoints[2] = GetTorusKnotPos( parallel / (float) ParallelSlices * 2.0f * M_PI * TorusKnotP, meridian + 1 );
    nearbyPoints[2] = torusCenter + (nearbyPoints[2] - torusCenter) * 0.3f + (nearbyPoints[2] - torusCenter) * 0.8f * 
                      (BumpTextureArray.SampleLevel( SamplerLinear, float3( GetTorusKnotTex( parallel, meridian + 1 ), (float)input.InstanceId ), 0 ));
    nearbyPoints[2] = mul( nearbyPoints[2], input.WorldTransform );

    nearbyPoints[3] = GetTorusKnotPos( (parallel - 1) / (float) ParallelSlices * 2.0f * M_PI * TorusKnotP, meridian );
    nearbyPoints[3] = torusCenter + (nearbyPoints[3] - torusCenter) * 0.3 + (nearbyPoints[3] - torusCenter) * 0.8f * 
                      (BumpTextureArray.SampleLevel( SamplerLinear, float3( GetTorusKnotTex( parallel - 1, meridian ), (float)input.InstanceId ), 0 ));
    nearbyPoints[3] = mul( nearbyPoints[3], input.WorldTransform );

    output.Norm = float3( 0.0f, 0.0f, 0.0f );

    // Finally, compute a normal

    for( int i = 0; i < 4; i ++ )
    {
        float3 edge1 = normalize( nearbyPoints[i] - output.Pos );
        float3 edge2 = normalize( nearbyPoints[(i + 1) % 4] - output.Pos );

        output.Norm += normalize( cross( edge1, edge2 ) );
    }

    output.Norm /= 4.0f;
    output.Norm = normalize( output.Norm );
    return output;
}

// Geometry Shader

[maxvertexcount(6)]
void GSTorus( triangle GSTorus_Input input[3], inout TriangleStream<PSTorus_Input> TriStream )
{
    PSTorus_Input output;

    for( int v = 0; v < 3; v ++ )
    {
        output.Pos = mul( input[v].Pos, ViewMatrix );
        output.Pos = mul( output.Pos, ProjectionMatrix );
        output.RealPos = input[v].Pos;
        output.Tex = input[v].Tex;
        output.Norm = input[v].Norm;

        TriStream.Append( output );
    }

    TriStream.RestartStrip();
}

// Pixel Shader

float4 PSTorus( PSTorus_Input input ) : SV_Target
{
    float4 lightCol;
    float4 lightDir;
    float4 eyeDir;
    float3 reflectedDir;
    
    // Perform diffuse and specular shading

    eyeDir = normalize( EyePosition - input.RealPos );
    lightDir = normalize( LightPosition - input.RealPos );
    reflectedDir = normalize( input.Norm * 2.0f * dot( (float3)lightDir, input.Norm ) - (float3)lightDir );
    lightCol = LightColor * (saturate( dot( (float3)lightDir, input.Norm ) ) + pow( saturate( dot( reflectedDir, (float3)eyeDir ) ), 15.0f ));

    return (DiffTextureArray.Sample( SamplerLinear, input.Tex ) * (lightCol + AmbientColor));
}

// Rendering shadows for torus knot
// Some structures

struct VSTorusShadows_Input
{
    row_major float4x4	WorldTransform	: WorldTransform;
    uint				VertexId		: SV_VertexID;
    uint				InstanceId		: SV_InstanceID;
};

struct GSTorusShadows_Input
{
    float4	Pos			: SV_POSITION;
};

struct PSTorusShadows_Input
{
    float4	Pos			: SV_POSITION;
    uint	RTIndex		: SV_RenderTargetArrayIndex;
    uint	VPIndex		: SV_ViewportArrayIndex;
};

// Vertex Shader
// Since we use this shader only for cumputing shadows, we dont need to compute
// its color and normal

GSTorusShadows_Input VSTorusShadows( VSTorusShadows_Input input )
{
    GSTorusShadows_Input output = (GSTorusShadows_Input) 0;
    float4 nearbyPoints[4], torusCenter;
    float3 texCoord;

    int parallel = input.VertexId % (ParallelSlices + 1),
	    meridian = input.VertexId / (ParallelSlices + 1);

    torusCenter = GetTorusKnotCenter( parallel / (float)ParallelSlices * 2.0f * M_PI * TorusKnotP, meridian );

    output.Pos = GetTorusKnotPos( parallel / (float)ParallelSlices * 2.0f * M_PI * TorusKnotP, meridian );
    texCoord.xy = GetTorusKnotTex( parallel, meridian );
    texCoord.z = (float)input.InstanceId;

    output.Pos = torusCenter + (output.Pos - torusCenter) * 0.3f + (output.Pos - torusCenter) * 0.8f * 
                 (BumpTextureArray.SampleLevel( SamplerLinear, texCoord, 0 ));
    output.Pos = mul( output.Pos, input.WorldTransform );

    return output;
}

// Geometry Shader
// for shadow map array

[maxvertexcount(18)]
void GSTorusShadowsArray( triangle GSTorusShadows_Input input[3], inout TriangleStream<PSTorusShadows_Input> TriStream )
{
    PSTorusShadows_Input output;
    
    // Render six times - for each slice of a cubemap
    
    for( int rt = 0; rt < 6; rt ++ )
    {
        for( int v = 0; v < 3; v ++ )
        {
            output.Pos = mul( input[v].Pos, CubeViewMatrices[rt] );
            output.Pos = mul( output.Pos, CubeProjectionMatrix );
            output.RTIndex = rt;
            output.VPIndex = 0;

            TriStream.Append( output );
        }

        TriStream.RestartStrip();
    }
}

// Pixel Shader

float4 PSTorusShadows( PSTorusShadows_Input input ) : SV_Target
{
    return float4( 1.0f, 1.0f, 1.0f, 1.0f );
}

// Rendering box
// Some structures

struct VSBox_Input
{
    uint2	MultiIndex	: TEXCOORD0;
};

struct GSBox_Input
{
    float4	Pos			: SV_POSITION;
    float4	RealPos		: TEXCOORD0;
    float2	Tex			: TEXCOORD1;
};

struct PSBox_Input
{
    float4	Pos			: SV_POSITION;
    float3	Norm		: TEXCOORD0;
    float3	Tex			: TEXCOORD1;
    float4	RealPos		: TEXCOORD2;
};

// Simplex noise functions
// Find which simplex we are in by magnitude sorting

void Simplex3D( const in float3 P, out float3 simplex[4] )
{
    float3 T = P.xzy >= P.yxz;
    simplex[0] = 0;
    simplex[1] = T.xzy > T.yxz;
    simplex[2] = T.yxz <= T.xzy;
    simplex[3] = 1;
}

// Hashing functions

int Hash( float3 P )
{
    //P /= 256;   // normalize texture coordinate
    //return PermTexture.SampleLevel( SamplerLinear, P.xy, 0 ).r ^ PermTexture.SampleLevel( SamplerLinear, float2( P.z, 0 ), 0 ).r;
	return PermTexture.Load( int3( P.xy, 0 ) ).r ^ PermTexture.Load( int3( P.z, 0, 0 ) ).r;
}

// gradient tables

// 3D gradients

const float3 Grad3[] = {
    float3( -1, -1, 0 ), // 0
    float3( -1, +1, 0 ), // 1
    float3( +1, -1, 0 ), // 2
    float3( +1, +1, 0 ), // 3

    float3( -1, 0, -1 ), // 4
    float3( -1, 0, +1 ), // 5
    float3( +1, 0, -1 ), // 6
    float3( +1, 0, +1 ), // 7

    float3( 0, -1, -1 ), // 8
    float3( 0, -1, +1 ), // 9
    float3( 0, +1, -1 ), // 10
    float3( 0, +1, +1 ), // 11

    // padding   
    float3( +1, +1, 0 ), // 12
    float3( -1, -1, 0 ), // 13
    float3( 0, -1, +1 ), // 14
    float3( 0, -1, -1 )  // 15
};

// Basic 3D simplex noise

#define F3 0.333333333333
#define G3 0.166666666667

float Snoise3D( float3 P )
{
    // Skew the (x,y,z) space to determine which cell of 6 simplices we're in
    
    float s = dot( P, F3 );
    float3 Pi = floor( P + s );
    float t = dot( Pi, G3 );

    float3 P0 = Pi - t;               // Unskew the cell origin back to (x,y,z) space
    float3 Pf0 = P - P0;            // The x,y,z distances from the cell origin

    // For the 3D case, the simplex shape is a slightly irregular tetrahedron.
    // To find out which of the six possible tetrahedra we're in, we need to
    // determine the magnitude ordering of x, y and z components of Pf0.

    float3 simplex[4];
    Simplex3D( Pf0, simplex );

    float n = 0;

    [unroll]
    for (int i = 0; i<4; i++)
    {
        float3 Pf = Pf0 - simplex[i] + G3 * i;
        int h = Hash( Pi + simplex[i] );
        float d = saturate( 0.6f - dot( Pf, Pf ) );
        d *= d; 
        n += d * d * dot(Grad3[ h & 31 ], Pf);
        //n += d * d * grad( Pf, h );
    }
    
    return 32.0f * n;
}

// Vertex Shader

GSBox_Input VSBox( VSBox_Input input )
{
    GSBox_Input output = (GSBox_Input) 0;
    
    // As an input to our vertex shader we've got a vector of two integers.
    // One integer is an index in a vertex buffer, and another integer is an index to 
    // texture coordinate buffer

    output.Pos = BoxVertexBuffer.Load( input.MultiIndex.x ).xyzw;
    output.RealPos = BoxVertexBuffer.Load( input.MultiIndex.x ).xyzw;
    output.Tex = BoxTexCoordBuffer.Load( input.MultiIndex.y ).xy;

    return output;
}

// Geometry Shader

[maxvertexcount(3)]
void GSBox( triangle GSBox_Input input [3], inout TriangleStream<PSBox_Input> TriStream, uint PrimitiveId : SV_PrimitiveID )
{
    PSBox_Input output;
    
    // Compute a normal to an input triangle

    float3 edge1 = normalize( input[1].Pos - input[0].Pos );
    float3 edge2 = normalize( input[2].Pos - input[0].Pos );
    float3 normal = normalize( cross( edge1, edge2 ) );

    // We don't have information about normals for each vertex, so we have to compute
    // normals in GS. Since we have a box, there is no need to make these normals smooth.
    // After we've computed the normals, we can transform triangle vertices to a screenspace.

    for (int v = 0; v < 3; v ++)
    {
        output.Pos = mul( input[v].Pos, ViewMatrix );
        output.Pos = mul( output.Pos, ProjectionMatrix );
        output.RealPos = input[v].RealPos;
        
        // We want a noise pattern to be different on each box face so
        // as a z-coordinate for texture coordinates we take a box face Id, which is a
        // primitive Id divided by 2. Division is made via shifting PrimitiveId right at one bit.

        output.Tex = float3( input[v].Tex.x, input[v].Tex.y, (float)(PrimitiveId >> 1) );

        output.Norm = normal;
        
        // Append transformed vertex

        TriStream.Append( output );
    }

    TriStream.RestartStrip();
}

// Pixel Shader
// for shadow map array

float4 PSBoxArray( PSBox_Input input ) : SV_Target
{
    float4 lightCol = float4( 0.0f, 0.0f, 0.0f, 0.0f );
    float4 lightDir = float4( 0.0f, 0.0f, 0.0f, 0.0f );
    float4 shadeColor = float4( 1.0f, 1.0f, 1.0f, 1.0f );
    float4 realPosLight = float4( 0.0f, 0.0f, 0.0f, 0.0f );
    float3 texCoord = float3( 0.0f, 0.0f, 0.0f );
    int cubeMapNum = 0;
    
    // Noise coordinate is a stretched texture coordinate. We want noise to change in time,
    // so we add Time variable to z-coordinate of a noise

    float3 noiseCoord = float3( (input.Tex.x - 0.5f) * 60.0f, (input.Tex.y - 0.5f) * 60.0f, input.Tex.z * 60.0f + Time * 3.0f );
    
    // Noise value is computed via turbulence function - a sum of weighted frequencies of Perlin noise

    float noise = 0.5f * (Snoise3D( noiseCoord * 0.03125f )) +
                  0.25f * (Snoise3D( noiseCoord * 0.0625f )) +
                  0.125f * (Snoise3D( noiseCoord * 0.125f )) +
                  0.0625f * (Snoise3D( noiseCoord * 0.25f )) +
                  0.03125f * (Snoise3D( noiseCoord * 0.5f )) +
                  0.015625f * (Snoise3D( noiseCoord * 1.0f ));
    
    // Use obtained noise value as a height value

    float3 offsetWorldPos = input.RealPos + input.Norm * noise * 8.0f;

    // We use ddx and ddy to find a normal to our bumped surface.
    // Instead of it we could try to find Perlin noise derivatives analytically or numerically,
    // which is difficult and slow. Using ddx and ddy is rather a cheap solution, however there remain
    // some visual artifacts due to nature of ddx and ddy (for each pixel they work only inside a block 2x2,
    // which contains this pixel)

    // One may doubt whether this approach gives correct results, because 
    // derivatives which we take using ddx and ddy, are taken along x- and y-coordinates in
    // screenspace, not in space of a surface. But for computing a normal to a surface it doesn't matter
    // whether we take an orthogonal pair of vectors or not. Until these vectors are not collinear we
    // will achieve correct results.

    float3 dx = ddx( offsetWorldPos );
    float3 dy = ddy( offsetWorldPos );
    float3 bumpNorm = normalize( cross( dx, dy ) );
    
    lightDir = normalize( LightPosition - float4( offsetWorldPos, 0.0f ) );
    float3 eyeDir = normalize( EyePosition - offsetWorldPos );
    float3 reflectedDir = normalize( bumpNorm * 2.0f * dot( (float3)lightDir, bumpNorm ) - (float3)lightDir );
    lightCol = LightColor * (saturate( dot( (float3)lightDir, bumpNorm ) ) + pow( saturate( dot( reflectedDir, eyeDir ) ), 15.0f ));
    
    // realPosLight is a current point position in light space

    realPosLight = - lightDir;
    
    // To decide from which surface of a cubemap we should fetch, first we should
    // find which coordinate has a biggest absolute value, and then check if it is
    // above or below zero.

    float maxCoord = max( abs( realPosLight.x ), max( abs( realPosLight.y ), abs( realPosLight.z ) ) );
        
    [flatten]
    if( maxCoord == abs( realPosLight.x ) )
    {
        cubeMapNum = realPosLight.x > 0 ? 0 : 1;
    }
    
    [flatten]
    if( maxCoord == abs( realPosLight.y ) )
    {
        cubeMapNum = realPosLight.y > 0 ? 2 : 3;
    }
    
    [flatten]
    if( maxCoord == abs( realPosLight.z ) )
    {
        cubeMapNum = realPosLight.z > 0 ? 4 : 5;
    }
    
    // Apply inverse transformations to a position in light space.

    realPosLight = float4( offsetWorldPos, 1.0f );
    realPosLight = mul( realPosLight, CubeViewMatrices[cubeMapNum] );
    realPosLight = mul( realPosLight, CubeProjectionMatrix );
    realPosLight *= (1.0f / abs( realPosLight.w ));
    
    // use a simple blurring filter to make soft shadows.

    float shadowSpace = 0.0f;

    for( int i = -1; i <= 1; i ++ )
    {
        for( int j = -1; j <= 1; j ++ )
        {
            // z-coordinate of texCoord is an index of cubemap face from which we should get
            // a depth-value corresponding to our point.

            texCoord = float3( (realPosLight.x / 2.0f) + 0.5f, (-realPosLight.y / 2.0f) + 0.5f, (float) cubeMapNum );

            if( realPosLight.z > ShadowMap.Sample( SamplerCube, texCoord, int2( i, j ) ).r )
            {
                shadowSpace += 1.0f / 9.0f;
            }
        }
    }
    
    // Compute the final color

    shadeColor -= float4( float3( 0.6f, 0.6f, 0.6f ) * shadowSpace, 1.0f );
    
    return (float4( 0.45f, 0.55f, 0.75f, 1.0f ) * (lightCol + AmbientColor) * shadeColor);
}

// Rasterizer state

RasterizerState MultisamplingEnabled
{
    MultisampleEnable = true;
	CullMode = FRONT;
	FrontCounterClockwise = true;
};

BlendState NoBlending
{
    BlendEnable[0] = FALSE;
};

DepthStencilState EnableDepth
{
    DepthEnable = true;
    DepthWriteMask = 1;
	DepthFunc = LESS;
};

// Technique for torus

technique10 RenderTorus
{
    pass P0
    {
		SetRasterizerState( MultisamplingEnabled );
		SetBlendState( NoBlending, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF );
		SetDepthStencilState( EnableDepth, 0 );

        SetVertexShader( CompileShader( vs_4_0, VSTorusShadows() ) );
        SetGeometryShader( CompileShader( gs_4_0, GSTorusShadowsArray() ) );
        SetPixelShader( CompileShader( ps_4_0, PSTorusShadows() ) );
    }

    pass P1
    {
		SetRasterizerState( MultisamplingEnabled );
		SetBlendState( NoBlending, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF );
		SetDepthStencilState( EnableDepth, 0 );

        SetVertexShader( CompileShader( vs_4_0, VSTorus() ) );
        SetGeometryShader( CompileShader( gs_4_0, GSTorus() ) );
        SetPixelShader( CompileShader( ps_4_0, PSTorus() ) );
    }
}

// Technique for box

technique10 RenderBox
{
    pass P0
    {
		SetRasterizerState( MultisamplingEnabled );
		SetBlendState( NoBlending, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF );
		SetDepthStencilState( EnableDepth, 0 );

        SetVertexShader( CompileShader( vs_4_0, VSBox() ) );
        SetGeometryShader( CompileShader( gs_4_0, GSBox() ) );
        SetPixelShader( CompileShader( ps_4_0, PSBoxArray() ) );
    }
}